from collections import OrderedDict

from django import forms
from django.utils.translation import ugettext_lazy as _

from .classic import SimpleClassicWithEmailSignup, SimpleClassicWithEmailSignupForm
from ..models import InvitationCode
from ..utils import USER


def get_username_max_len():
    """Returns username maximum length as supported by Django.

    :rtype: int
    """
    fields = [field for field in USER._meta.fields if field.name == 'username']
    try:
        length = fields[0].max_length
    except IndexError:
        length = 30
    return length


USERNAME_MAX_LEN = get_username_max_len()


class ModernSignupForm(SimpleClassicWithEmailSignupForm):
    """Modernized form with unique e-mail field, without username and second password fields."""

    def __init__(self, *args, **kwargs):
        super(ModernSignupForm, self).__init__(*args, **kwargs)
        if 'username' in self.fields:
            del self.fields['username']

    def clean_email(self):
        email = super(ModernSignupForm, self).clean_email()

        if USER._default_manager.filter(email__iexact=email):
            raise forms.ValidationError(_('A user with that e-mail already exists.'))

        if len(email) > USERNAME_MAX_LEN:
            raise forms.ValidationError(
                _('E-mail length should be no more than %(max_len)s characters long.'),
                params={'max_len': USERNAME_MAX_LEN})

        return email


class ModernSignup(SimpleClassicWithEmailSignup):
    """Modernized registration flow based on classic from Django built-in
    with unique e-mail field, without username and second password fields.

    """

    form = ModernSignupForm

    def sign_in(self, request, form, signup_result):
        form_data = form.cleaned_data
        return self.login_generic(request, form_data['email'], form_data['password1'])

    def add_user(self, request, form):
        user = super(form.__class__, form).save(commit=False)
        if not hasattr(user, 'USERNAME_FIELD') or user.USERNAME_FIELD == 'username':
            user.username = form.cleaned_data['email']
        user.email = form.cleaned_data['email']
        user.set_password(form.cleaned_data['password1'])
        user.save()
        self.send_email(request, user)
        return user


class InvitationSignupForm(ModernSignupForm):
    """Modernized form with invitation code field."""

    def __init__(self, *args, **kwargs):
        super(InvitationSignupForm, self).__init__(*args, **kwargs)
        try:
            self.fields.insert(0, 'code', forms.CharField(label=_('Invitation code')))
        except AttributeError:  # Django 1.7
            new_fields = OrderedDict()
            new_fields['code'] = forms.CharField(label=_('Invitation code'))
            for k, v in self.fields.items():
                new_fields[k] = v
            self.fields = new_fields

    def clean_code(self):
        code = self.cleaned_data['code']
        if not InvitationCode.is_valid(code):
            raise forms.ValidationError(_('This invitation code is invalid.'))
        return code


class InvitationSignup(ModernSignup):
    """Modernized registration flow with additional invitation code field."""

    form = InvitationSignupForm

    def add_user(self, request, form):
        user = super(InvitationSignup, self).add_user(request, form)
        InvitationCode.accept(form.cleaned_data['code'], user)
        return user



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Preferences'
        db.create_table('sitegate_preferences', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('signin_enabled', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('signin_disabled_text', self.gf('django.db.models.fields.TextField')(default='Sign in is disabled.')),
            ('signup_enabled', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('signup_disabled_text', self.gf('django.db.models.fields.TextField')(default='Sign up is disabled.')),
            ('signup_verify_email_title', self.gf('django.db.models.fields.CharField')(max_length=160, default='Account activation')),
            ('signup_verify_email_body', self.gf('django.db.models.fields.TextField')(default='Please follow the link below to activate your account:\n%(link)s')),
        ))
        db.send_create_signal('sitegate', ['Preferences'])

        # Adding model 'EmailConfirmation'
        db.create_table('sitegate_emailconfirmation', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(unique=True, max_length=128)),
            ('time_created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('time_accepted', self.gf('django.db.models.fields.DateTimeField')(null=True)),
            ('expired', self.gf('django.db.models.fields.BooleanField')(db_index=True, default=False)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['apps.User'])),
        ))
        db.send_create_signal('sitegate', ['EmailConfirmation'])


        # Changing field 'InvitationCode.acceptor'
        db.alter_column('sitegate_invitationcode', 'acceptor_id', self.gf('django.db.models.fields.related.ForeignKey')(null=True, to=orm['apps.User']))

        # Changing field 'InvitationCode.creator'
        db.alter_column('sitegate_invitationcode', 'creator_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['apps.User']))

    def backwards(self, orm):
        # Deleting model 'Preferences'
        db.delete_table('sitegate_preferences')

        # Deleting model 'EmailConfirmation'
        db.delete_table('sitegate_emailconfirmation')


        # Changing field 'InvitationCode.acceptor'
        db.alter_column('sitegate_invitationcode', 'acceptor_id', self.gf('django.db.models.fields.related.ForeignKey')(null=True, to=orm['auth.User']))

        # Changing field 'InvitationCode.creator'
        db.alter_column('sitegate_invitationcode', 'creator_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User']))

    models = {
        'apps.place': {
            'Meta': {'object_name': 'Place'},
            'geo_bounds': ('django.db.models.fields.CharField', [], {'null': 'True', 'max_length': '255', 'blank': 'True'}),
            'geo_pos': ('django.db.models.fields.CharField', [], {'null': 'True', 'max_length': '255', 'blank': 'True'}),
            'geo_title': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'geo_type': ('django.db.models.fields.CharField', [], {'null': 'True', 'db_index': 'True', 'blank': 'True', 'max_length': '25'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'raters_num': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'rating': ('django.db.models.fields.FloatField', [], {'db_index': 'True', 'default': '0'}),
            'status': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'}),
            'time_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'time_modified': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'auto_now': 'True', 'blank': 'True'}),
            'time_published': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'user_title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'apps.user': {
            'Meta': {'object_name': 'User'},
            'comments_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'disqus_category_id': ('django.db.models.fields.CharField', [], {'null': 'True', 'max_length': '30', 'blank': 'True'}),
            'disqus_shortname': ('django.db.models.fields.CharField', [], {'null': 'True', 'max_length': '100', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'to': "orm['auth.Group']", 'symmetrical': 'False', 'related_name': "'user_set'"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'place': ('django.db.models.fields.related.ForeignKey', [], {'null': 'True', 'related_name': "'users'", 'to': "orm['apps.Place']", 'blank': 'True'}),
            'raters_num': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'rating': ('django.db.models.fields.FloatField', [], {'db_index': 'True', 'default': '0'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'to': "orm['auth.Permission']", 'symmetrical': 'False', 'related_name': "'user_set'"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'to': "orm['auth.Permission']", 'symmetrical': 'False'})
        },
        'auth.permission': {
            'Meta': {'object_name': 'Permission', 'unique_together': "(('content_type', 'codename'),)", 'ordering': "('content_type__app_label', 'content_type__model', 'codename')"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'contenttypes.contenttype': {
            'Meta': {'object_name': 'ContentType', 'db_table': "'django_content_type'", 'unique_together': "(('app_label', 'model'),)", 'ordering': "('name',)"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'sitegate.blacklisteddomain': {
            'Meta': {'object_name': 'BlacklistedDomain'},
            'domain': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '253'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'db_index': 'True', 'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'sitegate.emailconfirmation': {
            'Meta': {'object_name': 'EmailConfirmation'},
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '128'}),
            'expired': ('django.db.models.fields.BooleanField', [], {'db_index': 'True', 'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time_accepted': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'time_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['apps.User']"})
        },
        'sitegate.invitationcode': {
            'Meta': {'object_name': 'InvitationCode'},
            'acceptor': ('django.db.models.fields.related.ForeignKey', [], {'null': 'True', 'related_name': "'acceptors'", 'to': "orm['apps.User']", 'blank': 'True'}),
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '128'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'creators'", 'to': "orm['apps.User']"}),
            'expired': ('django.db.models.fields.BooleanField', [], {'db_index': 'True', 'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time_accepted': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'time_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        },
        'sitegate.preferences': {
            'Meta': {'object_name': 'Preferences'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'signin_disabled_text': ('django.db.models.fields.TextField', [], {'default': "'Sign in is disabled.'"}),
            'signin_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'signup_disabled_text': ('django.db.models.fields.TextField', [], {'default': "'Sign up is disabled.'"}),
            'signup_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'signup_verify_email_body': ('django.db.models.fields.TextField', [], {'default': "'Please follow the link below to activate your account:\\n%(link)s'"}),
            'signup_verify_email_title': ('django.db.models.fields.CharField', [], {'max_length': '160', 'default': "'Account activation'"})
        }
    }

    complete_apps = ['sitegate']


#! /usr/bin/env python
import sys
import os

from django import VERSION
from django.conf import settings, global_settings


APP_NAME = 'sitegate'


def main():
    sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

    if not settings.configured:
        configure_kwargs = dict(
            INSTALLED_APPS=(
                'django.contrib.auth',
                'django.contrib.contenttypes',
                'django.contrib.sessions',
                'etc',
                APP_NAME,
            ),
            DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3'}},
            MIDDLEWARE_CLASSES=(
                'django.middleware.common.CommonMiddleware',
                'django.contrib.sessions.middleware.SessionMiddleware',
                'django.contrib.auth.middleware.AuthenticationMiddleware',
                'django.contrib.messages.middleware.MessageMiddleware',
            ),
            ROOT_URLCONF='sitegate.tests',
            MIGRATION_MODULES={},
            AUTH_USER_MODEL=os.environ.get('DJANGO_AUTH_USER_MODEL', 'auth.User')
        )

        if VERSION < (1, 11):
            configure_kwargs['MIGRATION_MODULES']['auth'] = 'django.contrib.auth.tests.migrations'

        try:
            configure_kwargs['TEMPLATE_CONTEXT_PROCESSORS'] = tuple(global_settings.TEMPLATE_CONTEXT_PROCESSORS) + (
                'django.core.context_processors.request',
            )

        except AttributeError:

            # Django 1.8+
            configure_kwargs['TEMPLATES'] = [
                {
                    'BACKEND': 'django.template.backends.django.DjangoTemplates',
                    'APP_DIRS': True,
                },
            ]

        settings.configure(**configure_kwargs)

    try:  # Django 1.7 +
        from django import setup
        setup()
    except ImportError:
        pass

    from django.test.utils import get_runner
    runner = get_runner(settings)()
    failures = runner.run_tests((APP_NAME,))

    sys.exit(failures)


if __name__ == '__main__':
    main()



from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.contrib.auth.forms import UserCreationForm
from django.contrib import messages

from .base import SignupFlow
from ..utils import USER
from ..models import BlacklistedDomain, EmailConfirmation
from ..settings import SIGNUP_VERIFY_EMAIL_BODY, SIGNUP_VERIFY_EMAIL_TITLE, SIGNUP_VERIFY_EMAIL_NOTICE, SIGNUP_VERIFY_EMAIL_VIEW_NAME


USERNAME_FIELD = getattr(USER, 'USERNAME_FIELD', 'username')


class ClassicSignupForm(UserCreationForm):

    """Classic form tuned to support custom user model."""

    error_messages = dict(UserCreationForm.error_messages, **{
        'duplicate_username': _('A user with that username already exists.'),
        'password_mismatch': _('The two password fields didn\'t match.'),
    })

    class Meta:
        model = USER
        fields = (USERNAME_FIELD,)

    def __init__(self, *args, **kwargs):
        super(ClassicSignupForm, self).__init__(*args, **kwargs)
        if USERNAME_FIELD != 'username' and 'username' in self.fields:
            del self.fields['username']

    def clean_username(self):
        username = self.cleaned_data['username']
        try:
            USER._default_manager.get(username=username)
        except USER.DoesNotExist:
            return username
        raise forms.ValidationError(self.error_messages['duplicate_username'])


class ClassicSignup(SignupFlow):
    """Classic registration flow borrowed from Django built-in UserCreationForm."""

    form = ClassicSignupForm

    def add_user(self, request, form):
        return form.save()

    def sign_in(self, request, form, signup_result):
        form_data = form.cleaned_data
        return self.login_generic(request, form_data['username'], form_data['password1'])


class SimpleClassicSignupForm(ClassicSignupForm):
    """Classic form without the second password field."""

    def __init__(self, *args, **kwargs):
        super(SimpleClassicSignupForm, self).__init__(*args, **kwargs)
        del self.fields['password2']


class SimpleClassicSignup(ClassicSignup):
    """Classic registration flow borrowed from Django built-in
    without second password field.

    """

    form = SimpleClassicSignupForm


class ClassicWithEmailSignupForm(ClassicSignupForm):
    """Classic form with email field."""

    email = forms.EmailField(label=_('Email'))

    class Meta:
        model = USER
        if USERNAME_FIELD != 'email':
            fields = (USERNAME_FIELD, 'email', 'password1', 'password2')
        else:
            fields = (USERNAME_FIELD, 'password1', 'password2')

    def __init__(self, *args, **kwargs):
        super(ClassicWithEmailSignupForm, self).__init__(*args, **kwargs)

    def clean_email(self):
        email = self.cleaned_data['email']
        if self.flow.get_arg_or_attr('validate_email_domain'):
            if BlacklistedDomain.is_blacklisted(email):
                raise forms.ValidationError(_('Sign Up with this email domain is not allowed.'))
        return email


class ClassicWithEmailSignup(ClassicSignup):
    """Classic registration flow borrowed from Django built-in
    with additional e-mail field.

    """

    form = ClassicWithEmailSignupForm
    validate_email_domain = True
    verify_email = False

    def __init__(self, **kwargs):
        super(ClassicWithEmailSignup, self).__init__(**kwargs)
        verify_email = self.get_arg_or_attr('verify_email')
        if verify_email:
            self.flow_args['activate_user'] = False
            self.flow_args['auto_signin'] = False

            try:
                from sitemessage.toolbox import get_message_type_for_app, schedule_messages, recipients

                def schedule_email(text, to, subject):
                    message_cls = get_message_type_for_app('sitegate', 'email_plain')
                    schedule_messages(message_cls(subject, text), recipients('smtp', to))

                self.schedule_email = schedule_email
            except ImportError as e:
                def schedule_email(text, to, subject):
                    send_mail(subject, text, settings.DEFAULT_FROM_EMAIL, [to.email])

                self.schedule_email = schedule_email

    def send_email(self, request, user):
        if getattr(self, 'schedule_email', False):
            code = EmailConfirmation.add(user)
            url = request.build_absolute_uri(reverse(SIGNUP_VERIFY_EMAIL_VIEW_NAME, args=(code.code,)))
            email_text = u'%s' % SIGNUP_VERIFY_EMAIL_BODY  # %s for PrefProxy objects.
            self.schedule_email(email_text % {'url': url}, user, u'%s' % SIGNUP_VERIFY_EMAIL_TITLE)
            messages.success(request, SIGNUP_VERIFY_EMAIL_NOTICE, 'info')

    def add_user(self, request, form):
        user = super(form.__class__, form).save(commit=False)
        user.set_password(form.cleaned_data['password1'])
        user.email = form.cleaned_data['email']
        user.save()
        self.send_email(request, user)
        return user


class SimpleClassicWithEmailSignupForm(ClassicWithEmailSignupForm):
    """Classic form with email field but without the second password field."""

    def __init__(self, *args, **kwargs):
        super(SimpleClassicWithEmailSignupForm, self).__init__(*args, **kwargs)
        del self.fields['password2']


class SimpleClassicWithEmailSignup(ClassicWithEmailSignup):
    """Classic registration flow borrowed from Django built-in
    with e-mail field, but without second password field.

    """

    form = SimpleClassicWithEmailSignupForm




from django import template, VERSION
from django.conf import settings

from ..exceptions import SiteGateError


register = template.Library()


_CONTEXT_FLATTEN = VERSION >= (1, 11)


class sitegate_flow_formNode(template.Node):
    """Base class for flow form nodes."""

    def __init__(self, flow_name):
        self.flow_name = flow_name

    def render(self, context):
        try:
            forms = context['request'].sitegate['%s_forms' % self.type]

            if self.flow_name is None:
                try:  # Try to use modern sign up.
                    flow_form = forms['ModernSignup']
                except KeyError:  # Try to get the first sign up flow form variable from request.
                    flow_form = forms[list(forms.keys())[0]]
            else:
                flow_form = forms[self.flow_name]
        except (AttributeError, KeyError):  # WSGI request attribute error; forms dict error.
            flow_form = None

        if flow_form is None:
            if settings.DEBUG:
                raise SiteGateError(
                    '`sitegate_%s_form` tag is used but the appropriate form is not found in context.' % self.type)
            return ''

        context.push()
        context['%s_form' % self.type] = flow_form
        content = template.loader.get_template(
            flow_form.template).render(context.flatten() if _CONTEXT_FLATTEN else context)

        return content


def tag_builder(parser, token, cls, flow_type):
    """Helper function handling flow form tags."""
    tokens = token.split_contents()
    tokens_num = len(tokens)

    if tokens_num == 1 or (tokens_num == 3 and tokens[1] == 'for'):
        flow_name = None
        if tokens_num == 3:
            flow_name = tokens[2]
        return cls(flow_name)
    else:
        raise template.TemplateSyntaxError(
            '"sitegate_%(type)s_form" tag requires zero or two arguments. '
            'E.g. {%% sitegate_%(type)s_form %%} or '
            '{%% sitegate_%(type)s_form for ClassicSignup %%}.' % {'type': flow_type})


@register.tag
def sitegate_signup_form(parser, token):
    """Renders signup forms."""
    return tag_builder(parser, token, sitegate_signup_formNode, 'signup')


@register.tag
def sitegate_signin_form(parser, token):
    """Renders signin forms."""
    return tag_builder(parser, token, sitegate_signin_formNode, 'signin')


class sitegate_signup_formNode(sitegate_flow_formNode):

    def __init__(self, flow_name):
        super(sitegate_signup_formNode, self).__init__(flow_name)
        self.type = 'signup'


class sitegate_signin_formNode(sitegate_flow_formNode):

    def __init__(self, flow_name):
        super(sitegate_signin_formNode, self).__init__(flow_name)
        self.type = 'signin'



from django.conf import settings
from django.utils.translation import ugettext_lazy as _


SIGNIN_ENABLED = getattr(settings, 'SITEGATE_SIGNIN_ENABLED', True)
SIGNIN_DISABLED_TEXT = getattr(settings, 'SITEGATE_SIGNIN_DISABLED_TEXT', _('Sign in is disabled.'))

SIGNUP_ENABLED = getattr(settings, 'SITEGATE_SIGNUP_ENABLED', True)
SIGNUP_DISABLED_TEXT = getattr(settings, 'SITEGATE_SIGNUP_DISABLED_TEXT', _('Sign up is disabled.'))

SIGNUP_VERIFY_EMAIL_NOTICE = getattr(
    settings, 'SITEGATE_SIGNUP_VERIFY_EMAIL_NOTICE',
    _('Congratulations! You\'re almost done with the registration. Please check your mailbox '
      'for a message containing an account activation link.'))

SIGNUP_VERIFY_EMAIL_TITLE = getattr(settings, 'SITEGATE_SIGNUP_VERIFY_EMAIL_TITLE', _('Account activation'))
SIGNUP_VERIFY_EMAIL_BODY = getattr(
    settings, 'SITEGATE_SIGNUP_VERIFY_EMAIL_BODY',
    _('Please follow the link below to activate your account:\n%(url)s'))

SIGNUP_VERIFY_EMAIL_SUCCESS_TEXT = getattr(
    settings, 'SITEGATE_SIGNUP_VERIFY_EMAIL_SUCCESS_TEXT',
    _('E-mail was successfully verified. You can now proceed to sign in.'))
SIGNUP_VERIFY_EMAIL_ERROR_TEXT = getattr(
    settings, 'SITEGATE_SIGNUP_VERIFY_EMAIL_ERROR_TEXT',
    _('Unable to verify an e-mail. User account was not activated.'))

SIGNUP_VERIFY_EMAIL_VIEW_NAME = getattr(settings, 'SITEGATE_SIGNUP_VERIFY_EMAIL_VIEW_NAME', 'verify_email')


if 'siteprefs' in settings.INSTALLED_APPS:
    from siteprefs.toolbox import patch_locals, register_prefs, pref, pref_group
    from django.db.models import CharField

    patch_locals()
    register_prefs(
        pref(SIGNIN_ENABLED, verbose_name=_('Signin enabled'), static=False),
        pref(SIGNIN_DISABLED_TEXT, verbose_name=_('Sign in disabled text'),
             help_text=_('This text is shown instead of a signin form when sign in in disabled.'), static=False),
        pref(SIGNUP_ENABLED, verbose_name=_('Signup enabled'), static=False),
        pref(SIGNUP_DISABLED_TEXT, verbose_name=_('Sign up disabled text'),
             help_text=_('This text is shown instead of a signup form when sign up in disabled.'), static=False),

        pref_group(_('E-mail verification'), (
            pref(SIGNUP_VERIFY_EMAIL_NOTICE, verbose_name=_('Verification notice'),
                 help_text=_('Text shown to a user after a registration form is submitted.'), static=False),

            pref(SIGNUP_VERIFY_EMAIL_TITLE, verbose_name=_('Message title'),
                 help_text=_('Title of an email message containing an account activation URL.'),
                 static=False, field=CharField(max_length=160)),
            pref(SIGNUP_VERIFY_EMAIL_BODY, verbose_name=_('Message body'),
                 help_text=_('<b>NOTE:</b> Message body must include `%(url)s` marker to indicate a place '
                             'to put an activation URL at.'), static=False),

            pref(SIGNUP_VERIFY_EMAIL_SUCCESS_TEXT, verbose_name=_('Verification success notice'),
                 help_text=_('A message shown to a user after he has followed an account activation URL '
                             'if activation was a success.'), static=False),
            pref(SIGNUP_VERIFY_EMAIL_ERROR_TEXT, verbose_name=_('Verification error notice'),
                 help_text=_('A message shown to a user after he has followed an account activation URL '
                             'if there was an error during an activation process.'), static=False),
        ))
    )



"""This file contains tests for sitegate."""
import unittest
from uuid import uuid4

try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User
    get_user_model = lambda: User


try:
    from django.http.response import HttpResponse
except ImportError:
    from django.http import HttpResponse

try:
    from django.contrib.auth.tests.custom_user import CustomUser
except ImportError:
    # Since 1.10
    pass

from django import VERSION as DJANGO_VERSION
from django.conf.urls import url
from django.core import urlresolvers
from django.core.urlresolvers import reverse, RegexURLPattern
from django.http import HttpResponseRedirect
from django.template.base import Template
from django.template.context import Context
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.utils import timezone

from sitegate.decorators import signin_view, signup_view, redirect_signedin, sitegate_view
from sitegate.signup_flows.classic import *
from sitegate.signup_flows.modern import *
from sitegate.signin_flows.classic import *
from sitegate.signin_flows.modern import *
from sitegate.models import *
from sitegate.toolbox import *
from sitegate.views import verify_email, messages
from sitegate.utils import USER, get_username_field


class MockUser(object):

    def __init__(self, logged_in):
        self.logged_in = logged_in

    def is_authenticated(self):
        return self.logged_in


@signup_view(activate_user=False, auto_signin=False, validate_email_domain=False, redirect_to='ok')
def register(request):
    return response_from_string(request, "{% load sitegate %}{% sitegate_signup_form %}")


@redirect_signedin('fail')
@signin_view(redirect_to='ok')
def login(request):
    return response_from_string(request, "{% load sitegate %}{% sitegate_signin_form %}")


urlpatterns = [
    url(r'entrance/', lambda r: None),
    url(r'^ok/$', lambda r: HttpResponse('ok'), name='ok'),
    url(r'^fail/$', lambda r: HttpResponse('fail'), name='fail'),
    url(r'^login/$', login, name='login'),
    url(r'^register/$', register, name='register'),
] + get_sitegate_urls()


if VERSION < (1, 10):
    from django.conf.urls import patterns
    urlpatterns.insert(0, '')
    urlpatterns = patterns(*urlpatterns)


def response_from_string(request, string):
    return HttpResponse(Template(string).render(Context({'request': request})))


URL_REGISTER = reverse('register')
URL_LOGIN = reverse('login')

MESSAGE_SUCCESS = None
MESSAGE_ERROR = None


def catch_message_success(*args, **kwargs):
    global MESSAGE_SUCCESS
    MESSAGE_SUCCESS = True


def catch_message_error(*args, **kwargs):
    global MESSAGE_ERROR
    MESSAGE_ERROR = True


# Mimic messages contrib.
messages.success = catch_message_success
messages.error = catch_message_error


@override_settings(USE_I18N=False)
class ViewsTest(TestCase):

    def setUp(self):
        # user credentials
        self._username = self._regenerate_email()
        self._password = 'qwerty'

    def _register_user(self, **kwargs):
        default_data = {'email': self._email, 'password1': self._password, "signup_flow": "ModernSignup"}
        default_data.update(kwargs)
        return self.client.post(URL_REGISTER, default_data)

    def _login(self, **kwargs):
        default_data = {'username': self._email, 'password': self._password, "signin_flow": "ModernSignin"}
        default_data.update(kwargs)
        return self.client.post(URL_LOGIN, default_data)

    def _regenerate_email(self, name_len=20):
        self._email = '{}@mail.com'.format('a' * name_len)
        return self._email

    @unittest.skipIf(settings.AUTH_USER_MODEL != 'auth.User', 'Custom user model is not supported for this test')
    def test_modern_signin(self):

        # username is too long
        if DJANGO_VERSION < (1, 5, 0):
            self._regenerate_email(200)
            response = self._login()
            self.assertEqual(response.status_code, 200)
            self.assertFormError(response, 'signin_form', 'username', 'Ensure this value has at most 30 characters'
                                                                      ' (it has %s).' % len(self._email))

        self._regenerate_email()

        # login with wrong credentials
        response = self._login()
        self.assertEqual(response.status_code, 200)

        if DJANGO_VERSION >= (1, 5, 0):  # 1.4 has html markup in actual errors description
            self.assertFormError(response, 'signin_form', None, 'Please enter a correct username and password. '
                                                                'Note that both fields may be case-sensitive.')

        # create deactivated test user
        response = self._register_user()
        self.assertRedirects(response, reverse('ok'))
        response = self._login()

        if DJANGO_VERSION < (1, 10, 0):
            # Since 1.10 this part is not even reached.
            self.assertFormError(response, 'signin_form', None, 'This account is inactive.')

        else:
            self.assertFormError(response, 'signin_form', None, 'Please enter a correct username and password. '
                                                                'Note that both fields may be case-sensitive.')

        # activate user and try to login again
        get_user_model().objects.filter(username=self._username).update(is_active=True)
        response = self._login()
        self.assertRedirects(response, reverse('ok'))

        # how about to login when user is already signed in?
        response = self._login()
        self.assertRedirects(response, reverse('fail'))
        self.client.logout()

        # more than one user with this e-mail
        get_user_model().objects.create(username='dummy', email=self._email)
        response = self._login()
        self.assertFormError(response, 'signin_form', None, 'There is more than one user with this e-mail. '
                                                            'Please use your username to log in.')

    @unittest.skipIf(settings.AUTH_USER_MODEL != 'auth.User', 'Custom user model is not supported for this test')
    def test_modern_signup(self):
        # create deactivated user
        response = self._register_user()
        self.assertRedirects(response, reverse('ok'))

        # auto_signin = False
        self.assertFalse(str(self.client.cookies).strip(), 'Cookies not empty')

        # he can't login yet
        login_result = self.client.login(username=self._username, password=self._password)
        self.assertFalse(login_result)

        # check fields
        user = get_user_model()._default_manager.get(username=self._username)
        self.assertEqual(user.email, self._email)
        self.assertEqual(user.username, self._username)

        # try to sign up with the same data for second time
        response = self._register_user()
        self.assertEqual(response.status_code, 200)
        self.assertFormError(response, 'signup_form', 'email', 'A user with that e-mail already exists.')

        # try to signup with email too long
        self._email = '%s@some.com' % ('long' * 100)
        response = self._register_user()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'E-mail length should be no more than')


class DecoratorsTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # set urlconf to test's one
        cls.urlconf_bak = urlresolvers.get_urlconf()
        urlresolvers.set_urlconf('sitegate.tests')

    @classmethod
    def tearDownClass(cls):
        urlresolvers.set_urlconf(cls.urlconf_bak)

    def setUp(self):
        self.req_factory = RequestFactory()

    def test_signin(self):
        # simple decoration
        response = signin_view(lambda req: req)(self.req_factory.get('/signin/'))
        self.assertTrue(hasattr(response, 'sitegate'))
        self.assertTrue('signin_forms' in response.sitegate)
        self.assertFalse('signup_forms' in response.sitegate)

        # simple decoration + view with args
        response = signin_view(lambda req, a1, a2='b': (req, a1, a2))(self.req_factory.get('/signin/'), 10, a2=20)
        self.assertTrue(hasattr(response[0], 'sitegate'))
        self.assertTrue('signin_forms' in response[0].sitegate)
        self.assertFalse('signup_forms' in response[0].sitegate)
        self.assertEqual(response[1], 10)
        self.assertEqual(response[2], 20)

        # stacking
        signin_view1 = signin_view(flow=ClassicSignin)
        signin_view2 = signin_view(flow=ModernSignin)
        response = signin_view1(signin_view2(lambda req: req))(self.req_factory.get('/signin/'))
        self.assertTrue(hasattr(response, 'sitegate'))
        self.assertTrue('signin_forms' in response.sitegate)
        self.assertFalse('signup_forms' in response.sitegate)
        self.assertTrue(len(response.sitegate['signin_forms']) == 2)
        dict_keys = list(response.sitegate['signin_forms'].keys())
        self.assertIsInstance(response.sitegate['signin_forms'][dict_keys[0]], ClassicSignin.form)
        self.assertIsInstance(response.sitegate['signin_forms'][dict_keys[1]], ModernSignin.form)

        # stacking with args
        signin_view1 = signin_view(flow=ClassicSignin, widget_attrs={'attr_name1': 'attr_value1'})
        signin_view2 = signin_view(flow=ModernSignin, widget_attrs={'attr_name2': 'attr_value2'})
        response = signin_view1(signin_view2(lambda req: req))(self.req_factory.get('/signin/'))
        self.assertTrue(hasattr(response, 'sitegate'))
        field = response.sitegate['signin_forms']['ClassicSignin'].fields.get('password')
        self.assertIn('attr_name1', field.widget.attrs)
        field = response.sitegate['signin_forms']['ModernSignin'].fields.get('password')
        self.assertIn('attr_name2', field.widget.attrs)

    def test_signup(self):
        # simple decoration
        response = signup_view(lambda req: req)(self.req_factory.get('/signup/'))
        self.assertTrue(hasattr(response, 'sitegate'))
        self.assertTrue('signup_forms' in response.sitegate)
        self.assertFalse('signin_forms' in response.sitegate)

        # simple decoration + view with args
        response = signup_view(lambda req, a1, a2='b': (req, a1, a2))(self.req_factory.get('/signup/'), 10, a2=20)
        self.assertTrue(hasattr(response[0], 'sitegate'))
        self.assertTrue('signup_forms' in response[0].sitegate)
        self.assertFalse('signin_forms' in response[0].sitegate)
        self.assertEqual(response[1], 10)
        self.assertEqual(response[2], 20)

        # stacking
        signup_view1 = signup_view(flow=ClassicSignup)
        signup_view2 = signup_view(flow=ModernSignup)
        response = signup_view1(signup_view2(lambda req: req))(self.req_factory.get('/signup/'))
        self.assertTrue(hasattr(response, 'sitegate'))
        self.assertTrue('signup_forms' in response.sitegate)
        self.assertFalse('signin_forms' in response.sitegate)
        self.assertTrue(len(response.sitegate['signup_forms']) == 2)
        dict_keys = list(response.sitegate['signup_forms'].keys())
        self.assertIsInstance(response.sitegate['signup_forms'][dict_keys[0]], ClassicSignup.form)
        self.assertIsInstance(response.sitegate['signup_forms'][dict_keys[1]], ModernSignup.form)

        # stacking with args
        signup_view1 = signup_view(flow=ClassicSignup, widget_attrs={'attr_name1': 'attr_value1'})
        signup_view2 = signup_view(flow=ModernSignup, widget_attrs={'attr_name2': 'attr_value2'})
        response = signup_view1(signup_view2(lambda req: req))(self.req_factory.get('/signup/'))
        self.assertTrue(hasattr(response, 'sitegate'))
        field = response.sitegate['signup_forms']['ClassicSignup'].fields.get('password1')
        self.assertIn('attr_name1', field.widget.attrs)
        field = response.sitegate['signup_forms']['ModernSignup'].fields.get('password1')
        self.assertIn('attr_name2', field.widget.attrs)

    def test_signinup(self):
        # simple decoration
        response = signin_view(signup_view(lambda req: req))(self.req_factory.get('/entrance/'))
        self.assertTrue(hasattr(response, 'sitegate'))
        self.assertTrue('signup_forms' in response.sitegate)
        self.assertTrue('signin_forms' in response.sitegate)

        # simple decoration + view with args
        response = signin_view(signup_view(lambda req, a1, a2='b': (req, a1, a2)))(self.req_factory.get('/entrance/'), 10, a2=20)
        self.assertTrue(hasattr(response[0], 'sitegate'))
        self.assertTrue('signup_forms' in response[0].sitegate)
        self.assertTrue('signin_forms' in response[0].sitegate)
        self.assertEqual(response[1], 10)
        self.assertEqual(response[2], 20)

        # stacking with args
        view_signin = signin_view(widget_attrs={'attr_name1': 'attr_value1'})
        view_signup = signup_view(widget_attrs={'attr_name2': 'attr_value2'})
        response = view_signin(view_signup(lambda req, a1, a2='b': (req, a1, a2)))(self.req_factory.get('/entrance/'), 10, a2=20)
        self.assertTrue(hasattr(response[0], 'sitegate'))
        self.assertTrue('signup_forms' in response[0].sitegate)
        self.assertTrue('signin_forms' in response[0].sitegate)
        self.assertEqual(response[1], 10)
        self.assertEqual(response[2], 20)
        field = response[0].sitegate['signin_forms']['ModernSignin'].fields.get('password')
        self.assertIn('attr_name1', field.widget.attrs)
        field = response[0].sitegate['signup_forms']['ModernSignup'].fields.get('password1')
        self.assertIn('attr_name2', field.widget.attrs)

    def test_redirect(self):
        # logged in
        # default redirect
        request = self.req_factory.get('/entrance/')
        request.user = MockUser(logged_in=True)
        response = redirect_signedin(lambda req: req)(request)
        self.assertIsInstance(response, HttpResponseRedirect)

        # not logged in
        request = self.req_factory.get('/entrance/')
        request.user = MockUser(logged_in=False)
        response = redirect_signedin(lambda req: req)(request)
        self.assertNotIsInstance(response, HttpResponseRedirect)

        # paametrized decoration
        request = self.req_factory.get('/entrance/')
        request.user = MockUser(logged_in=True)
        redirector = redirect_signedin('/abc/')
        response = redirector(lambda req: req)(request)
        self.assertIsInstance(response, HttpResponseRedirect)
        self.assertEqual(response['Location'], '/abc/')

    def test_sitegate(self):
        # logged in
        request = self.req_factory.get('/entrance/')
        request.user = MockUser(True)
        response = sitegate_view(lambda req: req)(request)
        self.assertIsInstance(response, HttpResponseRedirect)

        # not logged in

        # simple decoration
        request = self.req_factory.get('/entrance/')
        request.user = MockUser(False)
        response = sitegate_view(lambda req: req)(request)
        self.assertNotIsInstance(response, HttpResponseRedirect)
        self.assertTrue(hasattr(response, 'sitegate'))
        self.assertTrue('signup_forms' in response.sitegate)
        self.assertTrue('signin_forms' in response.sitegate)

        # simple decoration + view with args
        request = self.req_factory.get('/entrance/')
        request.user = MockUser(False)
        response = sitegate_view(lambda req, a1, a2='b': (req, a1, a2))(request, 10, a2=20)
        self.assertNotIsInstance(response, HttpResponseRedirect)
        self.assertTrue(hasattr(response[0], 'sitegate'))
        self.assertTrue('signup_forms' in response[0].sitegate)
        self.assertTrue('signin_forms' in response[0].sitegate)
        self.assertEqual(response[1], 10)
        self.assertEqual(response[2], 20)

        # parametrized decoration
        request = self.req_factory.get('/entrance/')
        request.user = MockUser(False)
        view_sitegate = sitegate_view(widget_attrs={'attr_name1': 'attr_value1'})
        response = view_sitegate(lambda req, a1, a2='b': (req, a1, a2))(request, 10, a2=20)
        self.assertTrue(hasattr(response[0], 'sitegate'))
        field = response[0].sitegate['signin_forms']['ModernSignin'].fields.get('password')
        self.assertIn('attr_name1', field.widget.attrs)
        field = response[0].sitegate['signup_forms']['ModernSignup'].fields.get('password1')
        self.assertIn('attr_name1', field.widget.attrs)
        self.assertEqual(response[1], 10)
        self.assertEqual(response[2], 20)


class ClassicWithEmailSignupTest(TestCase):

    @unittest.skipIf(settings.AUTH_USER_MODEL != 'auth.User', 'Custom user model is not supported for this test')
    def test_basic(self):
        flow = ClassicWithEmailSignup()
        self.assertTrue(flow.validate_email_domain)
        self.assertFalse(flow.verify_email)
        self.assertFalse(hasattr(flow, 'schedule_email'))

        flow = ClassicWithEmailSignup(verify_email=True)
        self.assertTrue(flow.flow_args['verify_email'])
        self.assertFalse(flow.flow_args['activate_user'])
        self.assertFalse(flow.flow_args['auto_signin'])
        self.assertTrue(hasattr(flow, 'schedule_email'))

        form = ClassicWithEmailSignupForm({
            'username': 'abcom',
            'email': 'a@b.com',
            'password1': 'qwerty',
            'password2': 'qwerty',
        })
        form.flow = flow

        def fake_schedule_email(text, user, title):
            self.assertEqual(user.username, 'abcom')
            self.assertEqual(title, 'Account activation')

        flow.schedule_email = fake_schedule_email

        new_user = flow.add_user(RequestFactory().get('/'), form)
        self.assertEqual(new_user.username, 'abcom')


class ClassicSignupFormsTest(unittest.TestCase):

    def test_classic_signup_attrs(self):
        f = ClassicSignupForm()
        fields = set(f.fields.keys())

        self.assertTrue(get_username_field() in fields)
        self.assertTrue('password1' in fields)
        self.assertTrue('password2' in fields)

    @unittest.skipIf(settings.AUTH_USER_MODEL != 'auth.User', 'Custom user model is not supported for this test')
    def test_classic_signup_validation(self):
        get_user_model()._default_manager.create(username='test_duplicate', email='test_duplicate@mail.some')

        f = ClassicSignupForm(data={
            'username': 'test_duplicate',
            'email': 'test_duplicate@mail.some',
        })
        f.full_clean()
        self.assertIn('username', f.errors)
        self.assertEqual(len(f.errors), 3)

        f = ClassicSignupForm(data={
            'username': 'test_duplicate',
            'email': 'test_duplicate@mail.some',
            'password1': 'password',
            'password2': 'password',
        })
        f.full_clean()
        self.assertIn('username', f.errors)
        self.assertEqual(len(f.errors), 1)

    def test_simple_classic_signup_attrs(self):
        f = SimpleClassicSignupForm()
        fields = set(f.fields.keys())

        self.assertTrue(get_username_field() in fields)
        self.assertTrue('password1' in fields)
        self.assertFalse('password2' in fields)

    def test_classic_with_email_signup_attrs(self):
        f = ClassicWithEmailSignupForm()
        fields = set(f.fields.keys())

        self.assertTrue(get_username_field() in fields)
        self.assertTrue('email' in fields)
        self.assertTrue('password1' in fields)
        self.assertTrue('password2' in fields)

    def test_simple_classic_with_email_signup_attrs(self):
        f = SimpleClassicWithEmailSignupForm()
        fields = set(f.fields.keys())

        self.assertTrue(get_username_field() in fields)
        self.assertTrue('email' in fields)
        self.assertTrue('password1' in fields)
        self.assertFalse('password2' in fields)


@unittest.skipIf(settings.AUTH_USER_MODEL == 'auth.User', 'Custom user model required')
class ClassicSignupCustomUserFormsTest(TestCase):

    def test_classic_signup_attrs(self):
        f = ClassicSignupForm()
        fields = set(f.fields.keys())

        self.assertTrue('email' in fields)
        self.assertTrue('password1' in fields)
        self.assertTrue('password2' in fields)

    def test_classic_signup_validation(self):
        get_user_model()._default_manager.create(
            email='test_duplicate@mail.some',
            date_of_birth=timezone.now().date(),
        )

        f = ClassicSignupForm(data={
            'email': 'test_duplicate@mail.some',
        })
        f.full_clean()
        self.assertIn('email', f.errors)
        self.assertEqual(len(f.errors), 3)

        f = ClassicSignupForm(data={
            'email': 'test_duplicate@mail.some',
            'password1': 'password',
            'password2': 'password',
        })
        f.full_clean()
        self.assertIn('email', f.errors)
        self.assertEqual(len(f.errors), 1)

    def test_simple_classic_signup_attrs(self):
        f = SimpleClassicSignupForm()
        fields = set(f.fields.keys())

        self.assertTrue('username' not in fields)
        self.assertTrue('email' in fields)
        self.assertTrue('password1' in fields)
        self.assertFalse('password2' in fields)

    def test_classic_with_email_signup_attrs(self):
        f = ClassicWithEmailSignupForm()
        fields = set(f.fields.keys())

        self.assertTrue('username' not in fields)
        self.assertTrue('email' in fields)
        self.assertTrue('password1' in fields)
        self.assertTrue('password2' in fields)

    def test_simple_classic_with_email_signup_attrs(self):
        f = SimpleClassicWithEmailSignupForm()
        fields = set(f.fields.keys())

        self.assertTrue('username' not in fields)
        self.assertTrue('email' in fields)
        self.assertTrue('password1' in fields)
        self.assertFalse('password2' in fields)



class ModernSignupFormsTest(unittest.TestCase):

    def test_modern_signup_attrs(self):
        f = ModernSignupForm()
        fields = set(f.fields.keys())

        self.assertFalse('username' in fields)
        self.assertTrue('email' in fields)
        self.assertTrue('password1' in fields)
        self.assertFalse('password2' in fields)

    def test_invitation_signup_attrs(self):
        f = InvitationSignupForm()
        fields = set(f.fields.keys())

        self.assertFalse('username' in fields)
        self.assertTrue('code' in fields)
        self.assertTrue('email' in fields)
        self.assertTrue('password1' in fields)
        self.assertFalse('password2' in fields)


class GenericSignupFlowTest(unittest.TestCase):

    def test_init_exception(self):
        self.assertRaises(NotImplementedError, SignupFlow)

    def test_getflow_name(self):
        self.assertEqual(ModernSignup.get_flow_name(), 'ModernSignup')


class ClassicSigninFormsTest(unittest.TestCase):

    def test_classic_signin_attrs(self):
        f = ClassicSigninForm()
        fields = set(f.fields.keys())

        self.assertTrue('username' in fields)
        self.assertTrue('password' in fields)


class ModernSigninFormsTest(unittest.TestCase):

    def test_modern_signin_attrs(self):
        f = ModernSigninForm()
        fields = set(f.fields.keys())

        self.assertTrue('username' in fields)
        self.assertTrue('password' in fields)


@unittest.skipIf(settings.AUTH_USER_MODEL != 'auth.User', 'Custom user model is not supported for this test')
class InvitationCodeModelTest(unittest.TestCase):

    def setUp(self):
        self.user = USER(username=str(uuid4()))
        self.user.save()

    def tearDown(self):
        self.user.delete()

    def test_generate_code(self):
        code = InvitationCode.generate_code()
        self.assertIsNotNone(code)

    def test_add(self):
        code = InvitationCode.add(self.user)
        self.assertIsNotNone(code)
        self.assertEqual(code.creator, self.user)
        self.assertIsNone(code.acceptor)
        self.assertIsNotNone(code.time_created)
        self.assertIsNone(code.time_accepted)
        self.assertFalse(code.expired)

    def test_accept(self):
        code = InvitationCode.add(self.user)
        rows = InvitationCode.accept(code.code, self.user)
        self.assertEqual(rows, 1)
        updated_code = InvitationCode.objects.get(code=code.code)
        self.assertEqual(updated_code.acceptor, self.user)
        self.assertIsNotNone(updated_code.time_accepted)
        self.assertTrue(updated_code.expired)


class BlacklistedDomainModelTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        BlacklistedDomain.objects.bulk_create([
            BlacklistedDomain(domain='denied1.com', enabled=False),
            BlacklistedDomain(domain='denied2.com', enabled=True),
            BlacklistedDomain(domain='denied3.com', enabled=True),
            BlacklistedDomain(domain='denied4.co.uk', enabled=True)
        ])

    @classmethod
    def tearDownClass(cls):
        BlacklistedDomain.objects.all().delete()

    def test_is_blacklisted(self):
        self.assertFalse(BlacklistedDomain.is_blacklisted('example1@denied1.com'))
        self.assertTrue(BlacklistedDomain.is_blacklisted('example2@denied2.com'))
        self.assertTrue(BlacklistedDomain.is_blacklisted('example3@some.denied3.com'))
        self.assertTrue(BlacklistedDomain.is_blacklisted('example3@some.denied3.COM'))
        self.assertFalse(BlacklistedDomain.is_blacklisted('example4@co.uk'))
        self.assertTrue(BlacklistedDomain.is_blacklisted('example4@denied4.co.UK'))
        self.assertTrue(BlacklistedDomain.is_blacklisted('example4@some.Denied4.co.UK'))


class ToolboxTest(unittest.TestCase):

    def test_get_sitegate_urls(self):
        urls = get_sitegate_urls()
        self.assertIsInstance(urls, list)
        self.assertTrue(len(urls) == 1)
        self.assertIsInstance(urls[0], RegexURLPattern)


@unittest.skipIf(settings.AUTH_USER_MODEL != 'auth.User', 'Custom user model is not supported for this test')
class ViewsModuleTest(unittest.TestCase):

    def test_verify_email(self):
        request = RequestFactory().get('/')

        code = 42
        result = verify_email(request, code)
        self.assertEqual(result._headers['location'][1], '/')
        self.assertTrue(MESSAGE_ERROR)

        user = USER(username=str(uuid4()))
        user.save()

        code = EmailConfirmation.add(user)
        result = verify_email(request, code, redirect_to='/here/')
        self.assertEqual(result._headers['location'][1], '/here/')
        self.assertTrue(MESSAGE_SUCCESS)



"""This file contains signals emitted by sitegate."""
import django.dispatch

# Emitted when user successfully signs up.
sig_user_signup_success = django.dispatch.Signal(providing_args=['signup_result', 'flow', 'request'])

# Emitted when user sign up fails.
sig_user_signup_fail = django.dispatch.Signal(providing_args=['signup_result', 'flow', 'request'])



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'InvitationCode'
        db.create_table('sitegate_invitationcode', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(max_length=128, unique=True)),
            ('time_created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('time_accepted', self.gf('django.db.models.fields.DateTimeField')(null=True)),
            ('creator', self.gf('django.db.models.fields.related.ForeignKey')(related_name='creators', to=orm['auth.User'])),
            ('acceptor', self.gf('django.db.models.fields.related.ForeignKey')(related_name='acceptors', to=orm['auth.User'], null=True, blank=True)),
            ('expired', self.gf('django.db.models.fields.BooleanField')(default=False, db_index=True)),
        ))
        db.send_create_signal('sitegate', ['InvitationCode'])


    def backwards(self, orm):
        # Deleting model 'InvitationCode'
        db.delete_table('sitegate_invitationcode')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '80', 'unique': 'True'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'ordering': "('name',)", 'db_table': "'django_content_type'", 'object_name': 'ContentType'},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'sitegate.invitationcode': {
            'Meta': {'object_name': 'InvitationCode'},
            'acceptor': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'acceptors'", 'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'code': ('django.db.models.fields.CharField', [], {'max_length': '128', 'unique': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'creators'", 'to': "orm['auth.User']"}),
            'expired': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time_accepted': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'time_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['sitegate']


"""Contains utility functions used by sitegate."""
from functools import wraps

from django.utils.decorators import available_attrs

# Custom User model - Django 1.4 backward compatibility.
try:
    from django.contrib.auth import get_user_model
    USER = get_user_model()
except ImportError:
    from django.contrib.auth.models import User
    USER = User
    get_user_model = lambda: USER

get_username_field = lambda: getattr(USER, 'USERNAME_FIELD', 'username')


class DecoratorBuilder(object):
    """Decorators builder. Facilitates decorators creation.
    Inherit from this and implement `handle` method.

    """

    def __init__(self, args, kwargs):
        """Accepts decoration function arguments."""
        self._args_dec = list(args)
        self._kwargs_dec = dict(kwargs)

    def handle(self, func, args_func, kwargs_func, args_dec, kwargs_dec):
        raise NotImplementedError('Please implement `handle` method.')

    def __call__(self, *args_call, **kwargs_call):
        def decorated(view_function):
            @wraps(view_function, assigned=available_attrs(view_function))
            def catcher(*args_func, **kwargs_func):
                return self.handle(view_function, args_func, kwargs_func, self._args_dec, self._kwargs_dec)
            return catcher

        # Case one: @dec('a', b='b')
        if len(args_call) and hasattr(args_call[0], '__call__'):
            return decorated(args_call[0])

        # Case two: @dec
        if len(self._args_dec) and hasattr(self._args_dec[0], '__call__'):
            return decorated(self._args_dec[0])(*args_call, **kwargs_call)

        return decorated
